using Microsoft.AspNetCore.Authentication.JwtBearer;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Hosting;
using Microsoft.Extensions.Logging;
using Microsoft.IdentityModel.Tokens;
using System;
using System.Collections.Generic;
using System.IdentityModel.Tokens.Jwt;
using System.Linq;
using System.Threading.Tasks;
using TEAMModelOS.Models;
using TEAMModelOS.SDK.Context.Configuration;
using TEAMModelOS.SDK.DI;
using TEAMModelOS.SDK.Models.Service;
using VueCliMiddleware;

namespace TEAMModeBI
{
    public class Startup
    {
        public IWebHostEnvironment environment { get; set; }
        readonly string MyAllowSpecificOrigins = "_myAllowSpecificOrigins";
        public Startup(IConfiguration configuration,IWebHostEnvironment env)
        {
            Configuration = configuration;
            environment = env;
            BaseConfigModel.SetBaseConfig(Configuration, env.ContentRootPath, env.WebRootPath);
        }

        public IConfiguration Configuration { get; }

        // This method gets called by the runtime. Use this method to add services to the container.
        public void ConfigureServices(IServiceCollection services)
        {
            // trueĬJr£ӳ䌢fʽӳQm^SAMLóRoleClaimType = 'http://schemas.microsoft.com/ws/2008/06/identity/claims/role'
            // falseRoleClaimType = 'roles'             
            JwtSecurityTokenHandler.DefaultMapInboundClaims = false;
            services.AddAuthentication(options => options.DefaultScheme = JwtBearerDefaults.AuthenticationScheme)
                .AddJwtBearer(options => //AzureADJwtBearer
                {
                    //options.SaveToken = true; //CɷɲЧm춷؆ֲʽܘ
                    options.Authority = Configuration["Option:Authority"];
                    options.Audience = Configuration["Option:Audience"];
                    options.RequireHttpsMetadata = true;
                    options.TokenValidationParameters = new TokenValidationParameters
                    {
                        RoleClaimType = "roles",
                        ValidAudiences = new string[] { Configuration["Option:Audience"], $"api://{Configuration["Option:Audience"]}" }
                    };
                    options.Events = new JwtBearerEvents();
                    //¼Ҫo䛄t_
                    //options.Events.OnMessageReceived = async context => { await Task.FromResult(0); };
                    //options.Events.OnForbidden = async context => { await Task.FromResult(0); };
                    //options.Events.OnChallenge = async context => { await Task.FromResult(0); };
                    //options.Events.OnAuthenticationFailed = async context => { await Task.FromResult(0); };
                    options.Events.OnTokenValidated = async context =>
                    {
                        if (!context.Principal.Claims.Any(x => x.Type.Equals("http://schemas.microsoft.com/identity/claims/scope")) //ClaimConstants.Scope
                        && !context.Principal.Claims.Any(y => y.Type.Equals("roles"))) //ClaimConstants.Roles //http://schemas.microsoft.com/ws/2008/06/identity/claims/role
                        {
                            //TODO ̎~ڙǽɫLo
                            throw new UnauthorizedAccessException("Neither scope or roles claim was found in the bearer token.");
                        }
                        await Task.FromResult(0);
                    };
                });
            //OՈ
            services.AddCors(options =>
            {
                options.AddPolicy(MyAllowSpecificOrigins,
                builder =>
                {
                    builder.WithOrigins("http://teammodelos-test.chinacloudsites.cn",
                                        "https://www.teammodel.cn", "https://localhost:5001",
                                        "http://localhost:5000")
                                       
                    .AllowAnyHeader()
                    .AllowAnyMethod();
                });
            });
            services.AddAzureStorage(Configuration.GetValue<string>("Azure:Storage:ConnectionString"));
            services.AddAzureRedis(Configuration.GetValue<string>("Azure:Redis:ConnectionString"));
            services.AddAzureCosmos(Configuration.GetValue<string>("Azure:Cosmos:ConnectionString"));
            services.AddAzureServiceBus(Configuration.GetValue<string>("Azure:ServiceBus:ConnectionString"));
            services.AddSnowflakeId(Convert.ToInt64(Configuration.GetValue<string>("Option:LocationNum")), 1);
            services.AddHttpClient();
            services.AddHttpClient<DingDing>();
            services.AddHttpClient<NotificationService>();
            services.AddMemoryCache();
            services.AddControllers().AddJsonOptions(options => { options.JsonSerializerOptions.IgnoreNullValues = false; });
            //HttpContextAccessorHttpContext(ṩMǿմȡHttpContext)
            services.AddHttpContextAccessor();
            services.Configure<Option>(options => Configuration.GetSection("Option").Bind(options));
            services.AddControllers();
            services.AddSpaStaticFiles(configuration =>
            {
                configuration.RootPath = "ClientApp";
            });
        }

        // This method gets called by the runtime. Use this method to configure the HTTP request pipeline.
        public void Configure(IApplicationBuilder app, IWebHostEnvironment env)
        {
            if (env.IsDevelopment())
            {
                app.UseDeveloperExceptionPage();
            }

            app.UseRouting();
            //Ҫdg  Ӧõ UseStaticFiles뽫 UseStaticFiles  UseRouting֮ǰ
            app.UseStaticFiles();
            app.UseSpaStaticFiles();
            app.UseCors(MyAllowSpecificOrigins); //ʹÿO
            app.UseHttpsRedirection(); //_lЕrP
            //Ӧʹ֤/Ȩܣ AuthorizePage  [Authorize]뽫 UseAuthentication  UseAuthorization
            //÷֮UseRouting  UseCors UseEndpoints֮ǰ
            app.UseAuthentication();
            app.UseAuthorization();

            app.UseEndpoints(endpoints =>
            {
                endpoints.MapControllers();
            });

            app.UseSpa(spa =>
            {
                if (env.IsDevelopment())
                    spa.Options.SourcePath = "ClientApp/";
                else
                    spa.Options.SourcePath = "dist";

                if (env.IsDevelopment())
                {
                    spa.UseVueCli(npmScript: "serve");
                }

            });
        }
    }
}
